<?php {
  /*
   *  dinolib page饹
   *  Υե 
   *   PukiWiki(http:// pukiwiki.org/) ̤ήѤƤޤ
   *   PukiWiki Copyright (C) 2001,2002 PukiWiki Developers Team,sng.
   *            License is GNU/GPL.
   *   $Id: page.php,v 1.13 2003/06/03 10:54:03 tumf Exp $
   *   This file is modified by Dino Co.,Ltd.
   */

   /**
    * dinolib page饹
    *
    * @access public
    * @package Dino_Page
    */
  require_once("link.php");

  class page {
    var $class_name="page";
    var $src;
    var $page_list=array();

    var $foot_explain;
    var $note_id;
    var $note_hr = '<hr />';
    var $strip_link_wall=true;

    var $html;

    var $user_rules=array();
    var $str_rules = array();
    var $line_rules = array();

    // եޡ롼
    // $usefacemark = 1ʤեޡִޤ
    // ʸXDʤɤäfacemarkִƤޤΤ
    // ɬפΤʤ $usefacemark0ˤƤ
    var $usefacemark = 1;
    var $facemark_rules = array();

    // ڡԽ¤뤫
    var $page_edit =false;

    //󥹥ȥ饯
    function page(){

      // 桼롼(ľܥִ)
      $this->str_rules = array(
			   "now\?" => date($date_format,UTIME)." (".$weeklabels[date("w",UTIME)].") ".date($time_format,UTIME),
			   "date\?" => date($date_format,UTIME),
			   "time\?" => date($time_format,UTIME),
			   );
    
      // 桼롼(СȻִľܤʤ)
      $this->line_rules
	= array(
		"COLOR\(([^\(\)]*)\){([^}]*)}" => "<span style=\"color:\\1\">\\2</span>",
		"SIZE\(([^\(\)]*)\){([^}]*)}" => "<span style=\"font-size:\\1px;display:inline-block;line-height:130%;text-indent:0px\">\\2</span>",
		"COLOR\(([^\(\)]*)\):((?:(?!COLOR\([^\)]+\)\:).)*)" => "<span style=\"color:\\1\">\\2</span>",
		"SIZE\(([^\(\)]*)\):((?:(?!SIZE\([^\)]+\)\:).)*)" => "<span class=\"size\\1\">\\2</span>",
		"LEFT:((?:(?!LEFT\:).)*)" => "<div style=\"text-align:left\">\\1</div>",
		"CENTER:((?:(?!CENTER\:).)*)" => "<div style=\"text-align:center\">\\1</div>",
		"RIGHT:((?:(?!RIGHT\:).)*)" => "<div style=\"text-align:right\">\\1</div>",
		"%%((?:(?!%%).)*)%%" => "<del>\\1</del>",
		"'''((?:(?!''').)*)'''" => "<em>\\1</em>",
		"''((?:(?!'').)*)''" => "<strong>\\1</strong>",
		"~((?:<\\/[a-zA-Z]+>)*)[\r]?$" => "\\1<br />", /* ˥ϲ */
		"&amp;aname\(([A-Za-z][\w\-]*)\);" => "<a id=\"\\1\" name=\"\\1\"></a>",
		);
    

      $this->facemark_rules 
	= array(
		"\s(\:\))" => " <img src=\"./face/smile.gif\" alt=\"\\1\" />",
		"\s(\:D)" => " <img src=\"./face/bigsmile.gif\" alt=\"\\1\" />",
		"\s(\:p)" => " <img src=\"./face/huh.gif\" alt=\"\\1\" />",
		"\s(\:d)" => " <img src=\"./face/huh.gif\" alt=\"\\1\" />",
		"\s(XD)" => " <img src=\"./face/oh.gif\" alt=\"\\1\" />",
		"\s(X\()" => " <img src=\"./face/oh.gif\" alt=\"\\1\" />",
		"\s(;\))" => " <img src=\"./face/wink.gif\" alt=\"\\1\" />",
		"\s(;\()" => " <img src=\"./face/sad.gif\" alt=\"\\1\" />",
		"\s(\:\()" => " <img src=\"./face/sad.gif\" alt=\"\\1\" />",
		"(\:heart\:)" => "<img src=\"./face/heart.gif\" alt=\"\\1\" />",
		);




      $this->user_rules=array(0);
      if($this->usefacemark) {
	$this->line_rules = array_merge($this->line_rules,$this->facemark_rules);
      }
      $this->user_rules = array_merge($this->str_rules,$this->line_rules);
    }


      // 󥯤ղä
      function make_link($name,$page = ''){
	$obj = new link_wrapper($page);
	$obj->page_list=$this->page_list;
	$obj->page_edit=$this->page_edit;
	return $obj->make_link($name);
      }
    
      // 
      function make_note($str){
	$str = preg_replace("/^\(\(/","",$str);
	$str = preg_replace("/\s*\)\)$/","",$str);

	$str= str_replace("\\'","'",$str);

	//$str = make_user_rules($str);

	$this->foot_explain[] 
	  = sprintf("<a name=\"notefoot_%s\" href=\"#notetext_%s\" class=\"note_super\">*%s</a> <span class=\"small\">$str</span><br />\n",
		    $this->note_id,
		    $this->note_id,
		    $this->note_id
		    );
		    
	$note
	  = sprintf("<a name=\"notetext_%s\" href=\"#notefoot_%s\" class=\"note_super\">*%s</a>",
			 $this->note_id,
			 $this->note_id,
			 $this->note_id
			 );
	$this->note_id++;

	return $note;
      }

      // 饤ǤΥѡ ()
      function inline($line,$remove=FALSE){
	$line = htmlspecialchars($line);
	
	$replace = $remove ? '' : '$this->make_note(\'$1\')';
	$line = preg_replace("/\(\(((?:(?!\)\)).)*)\)\)/ex",$replace,$line);
      
	return $line;
      }

      // 饤ǤΥѡ (󥯡ϢФ)
      function inline2($str){
	global $vars,$related,$related_link,$script;
	$cnts_plain = array();
	$arykeep = array();

	for($cnt=0;$cnt<count($str);$cnt++){
	  if(preg_match("/^(\s)/",$str[$cnt])){
	    $arykeep[$cnt] = $str[$cnt];
	    $str[$cnt] = "";
	    $cnts_plain[] = $cnt;
	  }
	}

	$str = $this->make_link($str);
	$str = preg_replace("/#related/e",'make_related($vars["page"],TRUE)',$str);
	$str = $this->make_user_rules($str);

	$tmp = $str;
	$str = preg_replace("/^#norelated$/","",$str);
	if($tmp != $str)
	  $related_link = 0;

	foreach($cnts_plain as $cnt){
	  $str[$cnt] = $arykeep[$cnt];
	}
	return $str;
      }


      // HTML
      function strip_htmltag($str){
	//$str = preg_replace("/<a[^>]+>\?<\/a>/","",$str);
	return preg_replace("/<[^>]+>/","",$str);
      }

      // 桼롼(ִС)
      function make_user_rules($str){
	foreach($this->user_rules as $rule => $replace) {
	  $str = preg_replace("/$rule/",$replace,$str);
	}
	return $str;
      }

      function list_push(&$result,&$saved,$tag,$level) {
	global $_list_left_margin, $_list_margin, $_list_pad_str;
	$cont = true;
	$open = "<$tag%s>";
	$close = "</li></$tag>";
	
	while (count($saved) > $level or
	       (count($saved) > 0 and $saved[0] != $close)) {
	  array_push($result, array_shift($saved));
	}
	
	$margin = $level - count($saved);
	
	while (count($saved) < ($level - 1)) {
	  array_unshift($saved, ''); //count($saved)䤹dummy
	}
	
	if (count($saved) < $level) {
	  $cont = false;
	  array_unshift($saved, $close);
		
	  $left = $margin * $_list_margin;
	  if ($level == $margin) $left += $_list_left_margin;
	  $str = sprintf($_list_pad_str, $level, $left, $left);
	  array_push($result, sprintf($open, $str));
	}
	
	if ($cont)
	  array_push($result, '</li>');
      }

      // $tagΥ$level٥ޤǵͤ롣
      function back_push(&$result,&$saved,$tag, $level){
	while (count($saved) > $level) {
		array_push($result, array_shift($saved));
	}
	if ($saved[0] != "</$tag>") {
		$result = array_merge($result,$saved); $saved = array();
	}
	while (count($saved) < $level) {
		array_unshift($saved, "</$tag>");
		array_push($result, "<$tag>");
	}
      }



      function convert(){
	$string = $this->src;
	$result=array();
	$saved=array();
	$table=0;

	$browser=new Dino_Browser;

	$lines = split("\n",$string);
	foreach ($lines as $line){
	  if(!preg_match("/^\/\/(.*)/",$line,$comment_out) && $table != 0){
	    if(!preg_match("/^\|(.+)\|[\r]?$/",$line,$out) or
	       $table != count(explode("|",$out[1]))){

	      $table = 0;
	      array_push($result, "</table></div>");
	    }
	  }

	  $comment_out = $comment_out[1];

	  // Ƭ񼰤ɤȽ
	  $line_head = substr($line,0,1);
	  if(	$line_head == ' ' || 
		$line_head == ':' || 
		$line_head == '>' || 
		$line_head == '-' || 
		$line_head == '+' || 
		$line_head == '|' || 
		$line_head == '*' || 
		$line_head == '#' || 
		$comment_out != ''
		) {
	    if(($headform[$_cnt-1] == '' || $headform[$_cnt-1] == '\r') && $_p){
	      array_push($result, "</p>");
	      $_p = FALSE;
	    }
	    if($line_head != '>' && $_bq){
	      array_push($result, "</p>");
	      $_bq = FALSE;
	    }

	    if(preg_match("/^\#([^\(]+)(.*)$/",$line,$out)){
	      // 	      if(exist_plugin_convert($out[1])) {
	      // 		$result = array_merge($result,$saved); $saved = array();
					
	      // 		if($out[2]) {
	      // 		  $_plugin = preg_replace("/^\#([^\(]+)\((.*)\)$/ex","do_plugin_convert('$1','$2')",$line);
	      // 		} else {
	      // 		  $_plugin = preg_replace("/^\#([^\(]+)$/ex","do_plugin_convert('$1','$2')",$line);
	      // 		}
	      // 		// Ƭ˶뤳ȤˤȤꤢpreΰƱͤinline2Ưޤ롢̵㡣
	      // 		array_push($result,"\t$_plugin");
	      // 	      } else {
	      //                array_push($result, htmlspecialchars($line));
	      //	      }
	    }else if(preg_match("/^(\*{1,3})(.*)/",$line,$out)){
	      $result = array_merge($result,$saved); $saved = array();
		$headform[$_cnt] = $out[1];
		$str = $this->inline($out[2]);
		$level = strlen($out[1]) + 1;

		$header_string="";
		if($browser->is_mobileagent()){
		  if($level==2){
		    $header_char="";
		    // ʸƬ*ˤϥѥ졼ʤ
		    if(count($result)>0){
		      $header_string="<hr><br>";  // not av <hr/>
		    }
		  }else if($level==3){
		    $header_char="";
		    $header_string="<br><br>";
		  }else{
		    $header_char="-";
		  }

		  $header_string
		    .="<h$level><a name=\"content_{$content_id_local}_$content_count\"></a>$header_char $str $top_link</h$level>";
		}else{
		  $header_string
		    ="<h$level><a name=\"content_{$content_id_local}_$content_count\"></a>$str $top_link</h$level>";
		}
		array_push($result, $header_string);

		$arycontents[] 
		  = str_repeat("-",$level-1)."<a href=\"#content_{$content_id_local}_$content_count\">"
		  .$this->strip_htmltag($this->make_user_rules($this->inline($out[2],TRUE)))
		  ."</a>\n";
		$content_count++;
	    }else if(preg_match("/^(-{1,4})(.*)/",$line,$out)){
	      $headform[$_cnt] = $out[1];
	      if(strlen($out[1]) == 4){
		$result = array_merge($result,$saved); $saved = array();
		array_push($result, "<hr />");
	      }else{
		$this->list_push($result,$saved,'ul', strlen($out[1]));
		array_push($result, '<li>'.$this->inline($out[2]));
	      }
	    }else if(preg_match("/^(\+{1,3})(.*)/",$line,$out)){
		$headform[$_cnt] = $out[1];
		$this->list_push($result,$saved,'ol', strlen($out[1]));
		array_push($result, '<li>'.$this->inline($out[2]));
		// }else if (preg_match("/^:([^:]+):(.*)/",$line,$out)){
	    }else if (preg_match("/^:(.*):(.*)/",$line,$out)){
		$headform[$_cnt] = ':'.$out[1].':';
		$this->back_push($result,$saved,'dl', 1);
		array_push($result, '<dt>' . $this->inline($out[1]) . '</dt>', '<dd>' . $this->inline($out[2]) . '</dd>');
	    }else if(preg_match("/^(>{1,3})(.*)/",$line,$out)){
		$headform[$_cnt] = $out[1];
		$this->back_push($result,$saved,'blockquote', strlen($out[1]));
		// ΤǼback_pushƤ봶̵
		if($headform[$_cnt-1] != $headform[$_cnt] ) {
		  if(!$_bq) {
		    array_push($result, "<p class=\"quotation\">");
		    $_bq = TRUE;
		  }else if(substr($headform[$_cnt-1],0,1) == '>'){
		    $_level_diff = abs( strlen($out[1]) - strlen($headform[$_cnt-1]) );
		    if( $_level_diff == 1 ){
		      $i = array_pop($result);
		      array_push($result, "</p>");
		      array_push($result,$i);
		      array_push($result, "<p class=\"quotation\">");
		      $_bq = TRUE;
		    } else {
		      $i = array();
		      $i[] = array_pop($result);
		      $i[] = array_pop($result);
		      array_push($result, "</p>");
		      $result = array_merge($result,$i);
		      array_push($result, "<p class=\"quotation\">");
		      $_bq = TRUE;
		    }
		  }
		}
		array_push($result, ltrim($this->inline($out[2])));
	    }else if(preg_match("/^(\s+.*)/",$line,$out)){
		$headform[$_cnt] = ' ';
		$this->back_push($result,$saved,'pre', 1);
		array_push($result, htmlspecialchars($out[1],ENT_NOQUOTES));
	    }else if(preg_match("/^\|(.+)\|[\r]?$/",$line,$out)){
		$headform[$_cnt] = '|';
		$arytable = explode("|",$out[1]);

		if(!$table){
		  $result = array_merge($result,$saved); $saved = array();
		  array_push($result,"<div class=\"ie5\"><table class=\"style_table\" cellspacing=\"1\" border=\"0\">");
		  $table = count($arytable);
		}

		array_push($result,"<tr>");
		foreach($arytable as $td){
		  if (preg_match("/^(LEFT|CENTER|RIGHT):(.*)$/",$td,$tmp)) {
		    $style = ' align="'.strtolower($tmp[1]).'"';
		    $td = $tmp[2];
		  } else {
		    $style = '';
		  }
		  array_push($result,"<td class=\"style_td\" $style>");
		  array_push($result,ltrim($this->inline($td)));
		  array_push($result,"</td>");
		}
		array_push($result,"</tr>");
	    }else if(strlen($comment_out) != 0){
	      $headform[$_cnt] = '//';
#				array_push($result," <!-- ".htmlspecialchars($comment_out)." -->");
	    }
	  } else {
	    $headform[$_cnt] = '';
	    if($headform[$_cnt-1] != $headform[$_cnt]){
	      if(array_values($saved)){
		if( $_bq ){
		  array_unshift($saved, "</p>");
		  $_bq = FALSE;
		}
		$i = array_pop($saved);
		array_push($saved,$i);
		$result = array_merge($result,$saved); $saved = array();
		//}if( substr($line,0,1) == '' && !$_p){
	      }if( preg_match("/^[\r]?$/",$line) && !$_p){
		array_push($result, "<p>");
		$_p = TRUE;
	      }else if( !preg_match("/^[\r]?$/",$line)  && $_p){
		array_push($result, "</p>");
		$_p = FALSE;
	      }
	    }
			
	    if (preg_match("/^(LEFT|CENTER|RIGHT):(.*)$/",$line,$tmp)) {
	      if ($_p)
		array_push($result,"</p>");
	      array_push($result,'<div align="'.strtolower($tmp[1]).'">');
	      array_push($result,$this->inline($tmp[2]));
	      array_push($result,"</div>");
	      $line = '';
	      $_p = FALSE;
	    }
	    if( preg_match("/^[\r]?$/",$line) && $_p){
	      $_tmp = array_pop($result);
	      if($_tmp == "<p>") {
		$_tmp = '<p class="empty">';
	      }
	      array_push($result, $_tmp, "</p>");
	      $_p = FALSE;
	    }else if(! preg_match("/^[\r]?$/",$line) && !$_p) {
	      array_push($result, "<p>");
	      $_p = TRUE;
	    }
	    if( !preg_match("/^[\r]?$/",$line) ){
	      array_push($result, $this->inline($line));
	    }

	  }
	  $_cnt++;
	}

	if($_p) array_push($result, "</p>");
	if($_bq) {
	  array_push($result, "</p>");
	}
	if($table) array_push($result, "</table></div>");
	
	$result_last = $result = array_merge($result,$saved); $saved = array();
	if($content_count != 0)	{
	  $result = array();
	  $saved = array();

	  foreach($arycontents as $line){
	    if(preg_match("/^(-{1,3})(.*)/",$line,$out)){
	      $this->list_push($result,$saved,'ul', strlen($out[1]));
	      array_push($result, '<li>'.$out[2]);
	    }
	  }
	  $result = array_merge($result,$saved); $saved = array();
		
	  $contents = "<a name=\"contents_$content_id_local\"></a>\n";
	  $contents .= join("\n",$result);
	  if($this->strip_link_wall){
	    $contents = preg_replace("/\[\[([^\]:]+):(.+)\]\]/","$1",$contents);
	    $contents = preg_replace("/\[\[([^\]]+)\]\]/","$1",$contents);
	  }
	}

	$result_last = $this->inline2($result_last);
	
	$result_last = preg_replace("/^#contents/",$contents,$result_last);

	$str = join("\n", $result_last);

	if ($this->foot_explain){
	  $str .= "\n<div class=\"footnote\">\n$this->note_hr\n";
	  $str .= join("\n",$this->inline2($this->foot_explain));
	  $str .= "\n</div>\n";
	}

	return $str;
      }

      function toBacketName($str){
	return rawurlencode("[[".$str."]]");
      }

      function fromBacketName($str){
	if(preg_match("/^\[\[(.*)\]\]$/",$str,$mm)){
	  return rawurldecode($mm[1]);
	}
	return $str;
      }


      function get_html(){
	return $this->html;
      }
      
  }
} ?>
